/*
 *   Copyright (C) 2006 
 *   Siraj Razick <siraj@kdemail.net>
 *   PhobosK <phobosk@mail.kbfx.org>
 *   see Also ATHORS 
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License version 2 as
 *   published by the Free Software Foundation
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "kbfxplasmalistbox.h"
#include "kbfxplasmapluginloader.h"

KbfxPlasmaListBox::KbfxPlasmaListBox( QWidget * parent , const char * name , WFlags l  ):
					QListBox(parent,name,Qt::WPaintUnclipped | Qt::WNoAutoErase | Qt::WStaticContents)
{
//this->viewport()->setWFlags(Qt::WNoAutoErase);

setFrameShape (QFrame::NoFrame);
m_pluginList =    KbfxPlasmaPluginLoader::scanPlugins(); 
this->setStaticBackground ( true) ;
this->setBackgroundMode(Qt::NoBackground);
this->viewport()->setBackgroundMode(Qt::NoBackground);

this->setVScrollBarMode(QScrollView::AlwaysOff);
this->setHScrollBarMode(QScrollView::AlwaysOff);


connect(this,SIGNAL(clicked(QListBoxItem*)),this,SLOT(slotClicked(QListBoxItem*)));
connect(this,SIGNAL(returnPressed(QListBoxItem*)),this,SLOT(slotClicked(QListBoxItem*)));
m_pressed = false;

}

KbfxPlasmaListBox::~KbfxPlasmaListBox()
{


}

void
KbfxPlasmaListBox::loadList(KbfxDataStack * stkPtr)
{

	if(stkPtr == NULL)
	{
		qDebug("Loading Stack Failed:KbfxListBox->loadList()");
			return ;
	}
	KbfxDataStack::DataDict _dict= stkPtr->getDataDict();
	KbfxDataStack::DataDict::Iterator it;


        for ( it =_dict.begin(); it != _dict.end(); ++it ) {

		KbfxPlasmaListBoxItem * item = new KbfxPlasmaListBoxItem(this);
		item->setKbfxText(it.data().name);
		item->setIcon(it.data().icon);
		item->setId(it.key());
        	item->setBelongsTo(stkPtr->name());
	}

}


/* DEPRICATED */
void
KbfxPlasmaListBox::loadList(KbfxPlasmaCanvasStack * stkPtr)
{

	if(stkPtr == NULL)
	{
		qDebug("Loading Stack Failed:KbfxListBox->loadList()");
			return ;
	}

	KbfxPlasmaCanvasStack::Dict _dict= stkPtr->getDict();
		
  KbfxPlasmaCanvasStack::Dict::Iterator it;

        for ( it =_dict.begin(); it != _dict.end(); ++it ) {

		KbfxPlasmaListBoxItem * item = new KbfxPlasmaListBoxItem(this);
	if(this->findItem(item->text()) != 0)
		continue;

		item->setKbfxText("&"+it.key());
		item->setId(it.data());
		item->setBelongsTo(stkPtr->name());
        	//connect(item,SIGNAL(raiseIt(uint)),stkPtr,SLOT(raise(uint)));
	}

}


void
KbfxPlasmaListBox::slotClicked(QListBoxItem* it)
{
 if(it == NULL)
	return ;
KbfxPlasmaListBoxItem * _it = (KbfxPlasmaListBoxItem*) it;
//	_it->mouseClicked();
	KbfxSignal signal(_it->belongsTo(),_it->id());

	clearSelection();
	_it->setMouseOver(true);
//printf("send signal from QListBox %d \n",signal.id());
//qDebug("Sending of text =-==> " + _it->text());	
	emit loadRequest(signal);
	this->setSelected(it,true);	
	if(m_pressed == true)
		m_pressed = false;
	if(m_pressed == false)
		m_pressed = true;

}

bool
KbfxPlasmaListBox::findLoaded(QString str)
{
	for (int i  = 0;i < count();i++)
	{	
		if( item(i) != NULL)
		{
				return true;
		
		}
	}
	
	return false;
}

void
KbfxPlasmaListBox::contentsContextMenuEvent(QContextMenuEvent *event)
{
m_pluginMenu = new KPopupMenu (this);;
connect(m_pluginMenu,SIGNAL(activated(int)),this,SLOT(contextMenuSlot(int)));
qDebug("Conext Menu Calls");
 m_pluginMenu->insertTitle ("KBFX Data Plugins",0,0);
 int _index = 1;

   for ( QStringList::Iterator it = m_pluginList.begin(); it != m_pluginList.end(); ++it ) {
	
	m_pluginMenu->insertItem(*it,_index,_index);
	_index++;
	}

 m_pluginMenu->exec(event->globalPos());

}

void
KbfxPlasmaListBox::contextMenuSlot(int id)
{
  	m_pluginList.remove(m_pluginList.at(id-1))   ;
	emit  pluginRequest(m_pluginMenu->text(id))  ;
}


void
KbfxPlasmaListBox::contentsMouseMoveEvent ( QMouseEvent * e ) 
{
	  QListBoxItem *_item = this->itemAt (e->pos ());
	if(_item == 0 )
	{
	return ;
	}
	else
	{
	KbfxPlasmaListBoxItem * _tmp = (KbfxPlasmaListBoxItem*)_item;
	//this->setSelected (_item, true);	
	if(_tmp !=0)
	{
	 clearAllButOne(_tmp);
		if(_tmp->getState() == false)
		{
		_tmp->setMouseOver(true);
		this->updateItem(_item);	
		}
	}		
	}
}



void
KbfxPlasmaListBox::clearAllButOne(KbfxPlasmaListBoxItem * item)
{
	
	if(item == 0)
		return ;

	for(int  i = 0 ; i < this->count() ; i++)
	{
	QListBoxItem * _item = this->item(i);
	if(_item == 0)
		continue;
	if(_item  == selectedItem())
	continue;

	KbfxPlasmaListBoxItem * __item = (KbfxPlasmaListBoxItem*) _item;

	if(__item->getState() == true)
	{
	__item->setMouseOver(false);
	this->updateItem(_item);
	}
}

}





void
KbfxPlasmaListBox::leaveEvent (QEvent * e)
{
  e = e;
//`    this->clearFocus ();
 for(int  i = 0 ; i < this->count() ; i++)
        {
        QListBoxItem * _item = this->item(i);
        if(_item == 0)
                continue;
	KbfxPlasmaListBoxItem * __item = (KbfxPlasmaListBoxItem*) _item;
        if(__item->isSelected() == false)
        {
        __item->setMouseOver(false);
        this->updateItem(_item);
        }

	}      
	
}


void
KbfxPlasmaListBox::clearItemSelection()
{
for(int  i = 0 ; i < this->count() ; i++)
        {
        QListBoxItem * _item = this->item(i);
        if(_item == 0)
                continue;
	KbfxPlasmaListBoxItem * __item = (KbfxPlasmaListBoxItem*) _item;
        if(__item->getState() == true)
        {
        __item->setMouseOver(false);
        this->updateItem(_item);
        }

	} 

}

void
KbfxPlasmaListBox::viewportPaintEvent(QPaintEvent * pe)
{

QWidget * vp = viewport();
//QPainter p (vp);
QRegion r = pe->region();


int x = contentsX();
int y = contentsY();
int w = vp->width();
int h = vp->height();

//int col = columnAt(x);
//int top = rowAt(y);




QPainter p ;

	QPixmap * buffer = new QPixmap(r.boundingRect().size());
	buffer->fill();
	p.begin(buffer);
	p.drawRect(pe->rect());
	p.fillRect(pe->rect(),QColor(255,255,255));
	int itemH=0;

	for(int i =  0 ; i < this->count() ; i++)
	{
	
		QListBoxItem * it  = this->item(i);
//		it->hide();
	}	

	for(int i = 0;i < this->numItemsVisible();i++)
	{
	
		QListBoxItem * it  = this->item(i);
//			it->show();	
	 KbfxPlasmaListBoxItem * _item = (KbfxPlasmaListBoxItem*) it;
		_item->setPos(0,itemH);
		_item->paint(&p);
		itemH += _item->height(this);
		

//		this->paintCell(&p,22,22);
	}
	p.end();
	bitBlt(this,0,0,buffer,0,0,pe->rect().width(),pe->rect().height());	
	delete buffer;

}

void
KbfxPlasmaListBox::keyPressEvent(QKeyEvent * ke)
{
	//qDebug("Key Pressed on Lisbox");
	if( ke->key() == Qt::Key_Down)
	{
		  QListBoxItem * it  = this->selectedItem();
                  QListBoxItem * it_next = it->next();
		 if(it_next != 0)
		{
	
		  KbfxPlasmaListBoxItem * _item = (KbfxPlasmaListBoxItem*) it_next;

		  clearAllButOne(_item);
		  this->setSelected(it_next,true);
		   _item->setMouseOver(true);
        	this->updateItem(_item);

		}		 

	}
	else if( ke->key() == Qt::Key_Up)
	{
	  QListBoxItem * it  = this->selectedItem();
                  QListBoxItem * it_prev = it->prev();
		 if(it_prev != 0)
		{
	
		  KbfxPlasmaListBoxItem * _item = (KbfxPlasmaListBoxItem*) it_prev;

		  clearAllButOne(_item);
		  this->setSelected(it_prev,true);
		   _item->setMouseOver(true);
        	this->updateItem(_item);

		}
	}
}

#include "kbfxplasmalistbox.moc"
