/*
 *   Copyright (C) 2006 
 *   Siraj Razick <siraj@kdemail.net>
 *   PhobosK <phobosk@mail.kbfx.org>
 *   see Also ATHORS 
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License version 2 as
 *   published by the Free Software Foundation
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "kbfxbutton.h"
#include <qtimer.h>

KbfxButton::KbfxButton(QWidget * parent, const char *name):
		       QLabel(parent,name,Qt::WStaticContents | Qt::WNoAutoErase |Qt::WPaintDesktop)
{
	  setLineWidth (0);
	  setScaledContents (false);

	//resize(0,0);
	loadSkins();
	m_toggle = false;
	QTimer * timer = new QTimer(this,"Update Timer");
	connect(timer,SIGNAL(timeout()),this,SLOT(update()));
//	timer->start(18,false);
	timer->start(25,false);

}

void
KbfxButton::loadSkins()
{
   this->readConfig();
   m_over_skin       =  QPixmap(ConfigInit ().m_KbfxHoverButtonPath);
   m_pressed_skin =  QPixmap(ConfigInit ().m_KbfxPressedButtonPath);
   m_normal_skin  =  QPixmap(ConfigInit ().m_KbfxNormalButtonPath);
   m_current_skin  =  m_normal_skin;
	qDebug(""+ConfigInit ().m_KbfxNormalButtonPath);   
   this->resize(m_current_skin.width(),m_current_skin.height());
   this->repaint();
}

void
KbfxButton::enterEvent ( QEvent * e)
{
	if(m_toggle == true)
	{
	m_current_skin = m_pressed_skin;
	}
	else
	{
	m_current_skin = m_over_skin;
	}
	this->resize(m_current_skin.size());
  this->update();
}

void
KbfxButton::leaveEvent (QEvent * e)
{	
	if(m_toggle)
	{
	m_current_skin = m_pressed_skin;
	}
	else
	{
         m_current_skin = m_normal_skin;
	}
	this->resize(m_current_skin.size());
	this->update();
}

void
KbfxButton::toggle()
{

	m_current_skin = m_normal_skin;
	this->resize(m_current_skin.size());
	this->update();
	if(!hasMouse())
		m_toggle = false;
}
KbfxButton::~KbfxButton()
{

}

void 
KbfxButton::mousePressEvent ( QMouseEvent * e ) 
{
	if(m_toggle == false)
	{
        m_current_skin = m_pressed_skin;
	m_toggle = true;
	this->resize(m_current_skin.size());
	this->update();
	if(e->button() == Qt::LeftButton)
	emit pressed();
	}
	else
	{
	m_current_skin = m_normal_skin;
	m_toggle = false;
	this->resize(m_current_skin.size());
	this->update();
	emit hideMenu();
	}
}

void
KbfxButton::readConfig()
{
 ConfigInit ().read ();
  ConfigInit().confskel->readConfig();
}

void
KbfxButton::paintEvent(QPaintEvent * pe)
{
  
QPainter p;
  if (!m_current_skin.isNull ())
    {

 static  const QRect r = QRect(pe->rect().x(),pe->rect().y(),m_current_skin.width(),m_current_skin.height());//pe->rect ();
  	m_buffer = new QPixmap();
  	m_buffer->resize(r.size());
  	m_buffer->fill(this, r.topLeft());

        p.begin (m_buffer,this);
        p.translate(-r.x(), -r.y());
        p.drawPixmap(r,m_current_skin);
        p.end ();
        bitBlt(this, r.x(), r.y(),m_buffer, 0, 0,m_current_skin.width(), m_current_skin.height());
	delete m_buffer;
    }

}

void
KbfxButton::setImage(QPixmap pixmap)
{
}
#include "kbfxbutton.moc"
